package tpdv;

/**
 * Classe que implementa um Terminal Ponto De Venda (TPDV).
 * Um TPDV  usado para fazer uma venda (uma nica venda de cada vez).
 * Itens podem ser comprados at o final da venda.
 * Um pagamento pode ser feito para a venda corrente.
 *
 * @author Craig Larman, Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.0
 */
class TPDV implements ITPDV {
    /**
     * O catlogo de produtos que podem ser vendidos neste TPDV.
     */
    private ICatalogoDeProdutos catlogo;
    /**
     * A venda corrente sendo realizada no TPDV.
     */
    private Venda venda;

    /**
     * Cria um TPDV.
     * @param catlogo Um catlogo de produtos que podem ser adquiridos neste TPDV.
     */
    public TPDV(ICatalogoDeProdutos catlogo) {
        this.catlogo = catlogo;
        venda = null;
    }

    /**
     * Obtm a venda corrente sendo realizada no TPDV.
     * @return A venda corrente sendo realizada no TPDV.
     */
    public IVenda getVenda() {
        return venda;
    }

    /**
     * Quando chamado, indica que a venda corrente sendo realizada no TPDV terminou.
     */
    public void fimDeVenda() throws NaoHaVendaException {
        if(venda == null) {
            throw new NaoHaVendaException("Nao ha venda iniciada");
        }
        venda.terminou();
    }

    /**
     * Informa um produto e a quantidade de itens deste produto sendo comprados na venda corrente.
     * Caso a venda anterior j tenha terminado, uma nova Venda  criada.
     * @param upc O Universal Product Code (UPC) do produto sendo comprado.
     * @param quantidade A quantidade de itens sendo comprados.
     */
    public void entraItem(int upc, int quantidade) throws ProdutoInexistenteException {
        if(isNovaVenda()) {
            venda = makeVenda();
        }
        venda.criaLinhaDetalhe(catlogo.getEspecificao(upc), quantidade);
    }

    /**
     * Realiza um pagamento para a venda corrente do TPDV.
     * @param valorEntregue O valor entregue pelo cliente para pagar a venda.
     */
    public void faaPagamento(float valorEntregue) throws PagamentoInsuficienteException {
        venda.faaPagamento(valorEntregue);
    }

    // visibilidade de package para poder testar
    boolean isNovaVenda() {
        return venda == null || venda.isTerminada();
    }

    // factory method
    protected Venda makeVenda() { return new Venda(); }
}
