package tpdv;

import java.util.*;

/**
 * Classe que representa uma venda de produtos feita atravs de um TPDV.
 * Uma venda  composta de vrias linhas de detalhe.
 * Enquanto a venda no terminou, tais linhas de detalhe podem ser criadas.
 * Um pagamento pode ser feita para pagar a venda.
 * Pode-se calcular o troco a ser entregue ao cliente.
 *
 * @author Craig Larman, Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.0
 */
class Venda implements IVenda {
    /**
     * As linhas de detalhe da venda.
     */
    private List linhasDetalhe = new Vector();
    /**
     * A data da venda.
     */
    private Date data = new Date(); // hoje
    /**
     * Indica se a venda terminou.
     */
    private boolean isTerminada = false;
    /**
     * O pagamento efetuado para a venda.
     */
    private IPagamento pagamento;

    /**
     * Calcula o valor total da venda.
     * @return O valor total da venda.
     */
    public float total() {
        float total = (float)0.0;
        Iterator it = linhasDetalhe.iterator();
        while(it.hasNext()) {
            total += ((ILinhaDetalhe)it.next()).subTotal();
        }
        return total;
    }

    /**
     * Calcule o troco para a venda, aps um pagamento.
     * @return O troco para a venda.
     */
    public float getTroco() {
        return pagamento.getValor() - total();
    }

    /**
     * Chamado para indicar que a venda terminou.
     */
    void terminou() {
        isTerminada = true;
    }

    /**
     * Obtm o status da venda.
     * @return true se a venda terminou; false caso contrrio.
     */
    boolean isTerminada() {
        return isTerminada;
    }

    /**
     * Cria uma linha de detalhe para a venda.
     * @param espec A especificao do produto sendo comprado.
     * @param quantidade A quantidade de itens (do mesmo produto) sendo comprados.
     */
    void criaLinhaDetalhe(IEspecProduto espec, int quantidade) {
        linhasDetalhe.add(makeLinhaDetalhe(espec, quantidade));
    }

    /**
     * Faz um pagamento para a venda.
     * @param valorEntregue O valor entregue pelo cliente para pagar a venda.
     */
    void faaPagamento(float valorEntregue) throws PagamentoInsuficienteException {
        if(valorEntregue < total()) {
            throw new PagamentoInsuficienteException("Pagamento insuficiente");
        }
        pagamento = makePagamento(valorEntregue);
    }

    // factory methods
    protected ILinhaDetalhe makeLinhaDetalhe(IEspecProduto espec, int quantidade) {
        return new LinhaDetalheVenda(espec, quantidade);
    }

    protected IPagamento makePagamento(float valorEntregue) {
        return new Pagamento(valorEntregue);
    }
}
